#!/usr/bin/env python2
# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

""" koji module of flr """

import os
import flr
import flr.util
import shutil
import tempfile

from dockerfile_parse import DockerfileParser

# Base koji command
KOJI_CMD = "koji"


def move_builds(src_tag, dest_tag, builds):
    """
    Move a build between two different tags in koji

    :param src_tag: str, the source/origin koji tag
    :param dest_tag: str, the destination koji tag
    :param builds: list of str, a list of one or many koji builds
    """

    # Compose the koji command
    cmd = [
        KOJI_CMD,
        "move-build",
        src_tag,
        dest_tag,
        ' '.join(builds),
    ]

    # Execute the koji command
    flr.util.subproc_call(
        cmd,
        log_msg="Running command: {0}".format(cmd)
    )


def latest_build(tag, pkgs):
    """
    Get a list from koji of the latest build for one or many packages

    :param tag: str, the koji tag to query for the latest package
    :param pkgs: list of str, a list of one or many packages to query for
    """

    # Compose the koji command
    cmd = [
        KOJI_CMD,
        "-q latest-build",
        tag,
        ' '.join(pkgs)
    ]

    # Execute the koji command
    # Capture the output
    stdout, stderr = flr.util.subproc_popen(
        cmd,
        log_msg="Running command: {0}".format(cmd)
    )

    # Parse the output and return the first element of each line (build name)
    return [x.split()[0] for x in stdout.strip().split('\n')]


def import_signatures(pkg_paths):
    """
    Import a signatures into koji

    :param pkg_paths: list of str, list of paths to signed rpms who's
        signatures need importing
    """

    # Bash equivalent
    #
    # koji import-signature  tmp/signed/*

    # Compose the koji command
    cmd = [
        KOJI_CMD,
        "import-sig",
        ' '.join(pkg_paths)
    ]

    # Execute the koji command
    flr.util.subproc_call(
        cmd,
        log_msg="Running command: {0}".format(cmd)
    )


def write_signed_rpms(key_sig, builds):
    """
    Write our signed rpms from koji

    :param key_sig: str, koji key signature
    :param builds: list of str, list of koji builds n-v-r
    """

    # Bash equivalent
    #
    # koji write-signed-rpm 81b46521 openh264-1.5.2-0.3.git21e44bd.fc24

    # Compose the koji command
    cmd = [
        KOJI_CMD,
        "write-signed-rpm",
        key_sig,
        ' '.join(builds)
    ]

    # Execute the koji command
    flr.util.subproc_call(
        cmd,
        log_msg="Running command: {0}".format(cmd)
    )


def rebuild(build_type, branch, anames, user=None, stage=False):
    """
    rebuild an artifact of type build_type
        Valid options are:
            - rpm
            - container

    :param build_type: str, type of artifact to rebuild
    :param branch: str, DistGit branch
    :param anames: tuple, artifact names (package names, container names, etc)
    :param user: str, FAS username (default: None)
    :param stage: bool, True == use stage, False = no (default: False)

    """
    # Set username if one is provided
    if user:
        user_opt = ("--user", user)
    else:
        user_opt = ("", "")

    # Set fedpkg command based on stage or not
    if stage:
        fedpkg_prefix = ["fedpkg-stage", user_opt[0], user_opt[1]]
        koji_prefix = ["stg-koji", user_opt[0], user_opt[1]]
    else:
        fedpkg_prefix = ["fedpkg", user_opt[0], user_opt[1]]
        koji_prefix = ["koji", user_opt[0], user_opt[1]]


    if build_type in ['rpm', 'rpms']:
        flr.log.error(
            "Not yet implemented, utility only supports containers at this time"
        )

    elif build_type in ['container', 'docker']:

        # Currently the container namespace in DistGit is called "docker"
        # but there's plans to change that in the future.
        cntr_ns = 'docker'


        work_dir = tempfile.mkdtemp()

        # List of koji tasks to watch at the end
        koji_tasks = []

        # Steps to build:
        #
        #   - Clone the DistGit Repo
        #   - Increment the Release
        #   - Write the Dockerfile out
        #   - fepkg commit
        #   - Open the Dockerfile
        #

        # Change directories to where we can do some work
        os.chdir(work_dir)

        for aname in anames:

            # Clone the DistGit repo
            cmd = fedpkg_prefix + [ 'clone', '{}/{}'.format(cntr_ns, aname)]
            # Execute the koji command
            flr.util.subproc_call(
                cmd,
                log_msg="Cloning {}/{} distgit repo".format(cntr_ns, aname)
            )

            # Change dirs into the DistGit repo
            flr.log.info("Changing dirs into {}/{}".format(work_dir, aname))
            os.chdir(os.path.join(work_dir, aname))

            # Switch branch to requested DistGit branch to operate on
            cmd = fedpkg_prefix + [ 'switch-branch', branch ]
            flr.util.subproc_call(
                cmd,
                log_msg="Switching branch to {}".format(branch)
            )

            # Load the Dockerfile content
            flr.log.info("Loading Dockerfile content")
            dfp = DockerfileParser()

            with open('Dockerfile', 'r') as dkr_file:
                dfp.content = ''.join(dkr_file.readlines())

            # Bump the RELEASE ENV var for the rebuild
            flr.log.info("Bumping RELEASE in Dockerfile")
            dfp_release = dfp.envs['RELEASE']
            # This is a weird hack, but it works and takes into account that we
            # don't want 0.9 + 0.1 to "wrap" to 1.0
            dfp_relsplit = dfp_release.split('.')
            dfp_newrel = dfp_relsplit[:-1]
            dfp_newrel.append(str(int(dfp_relsplit[-1]) + 1))
            dfp.envs['RELEASE'] = u'{}'.format('.'.join(dfp_newrel))

            # commit changes
            cmd = fedpkg_prefix + [
                'commit', '-m', 'Bump RELEASE for automatic rebuild'
            ]
            flr.util.subproc_call(
                cmd,
                log_msg="Committing changes to DistGit for {} Dockerfile".format(aname)
            )

            # push changes
            cmd = fedpkg_prefix + [ 'push' ]
            flr.util.subproc_call(
                cmd,
                log_msg="Pushing changes to DistGit for {} Dockerfile".format(aname)
            )

            # Build the image
            cmd = fedpkg_prefix + [ 'container-build', '--nowait' ]
            stdout, stderr = flr.util.subproc_popen(
                cmd,
                log_msg="Submitting koji build for {}".format(aname)
            )

            # Parse output for task id so we can wait for them later
            for line in stdout.split('\n'):
                if 'Created task' in line:
                    koji_tasks.append(line.split()[-1])

            # Clean up after ourselves
            os.chdir(work_dir)
            shutil.rmtree(os.path.join(work_dir, aname))


        # Wait for the koji tasks
        flr.log.info("Waiting for koji tasks to complete.")
        cmd = koji_prefix + ['watch-task']
        cmd.extend(koji_tasks)
        flr.util.subproc_call(cmd)



# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4
